import urllib
from gtkeasyapp.log import Log

##FoxyTag exception
class FoxyTagException(Exception):
	##constructor
	def __init__(self, message):
		self.message = message
	
	##To string
	def __str__(self):
		return 'FoxyTagException : ' + str(message)
		
##FoxyTag class
class FoxyTag():
	##constant
	FOXYTAG_FIXED = 'F'
	FOXYTAG_MOBILE = 'M'
	FOXYTAG_GHOST = 'G'
	FOXYTAG_CANCEL = 'C'

	##constructor
	##@param tag : tag string to split
	def __init__(self, tag=''):
		#init members
		self.lat = 0.0
		self.long = 0.0
		self.kind = self.FOXYTAG_FIXED
		self.heading = 0 #in degrees
		self.opposite = False
	
		if len(tag)>0:
			#split specified tag
			(self.lat, self.long, self.kind, self.heading) = tag.split('\t')
			self.lat = float(self.lat)
			self.long = float(self.long)
			self.heading = int(self.heading)
			#remove opposite info
			if self.heading>1000:
				self.heading -= 1000
		else:
			#reset to default values
			self.lat = 0
			self.long = 0
			self.kind = self.FOXYTAG_FIXED
			self.heading = 0
			
	##To string
	def __str__(self):
		return 'FOXYTAG(lat=%.5f, long=%.5f, kind=%s, heading=%d, opposite=%s)' % (self.lat, self.long, self.kind, self.heading, self.opposite)


##FoxyMsg class
class FoxyMsg():
	##Constructor
	def __init__(self, kind, line):
		self.kind = kind
		kind, self.message = line.split('\t')
		
##Foxy class
class Foxy():
	##Constants
	__FOXYTAG_HOST = 'http://www.foxytag.com/php/dogetlab.php'
	__FOXYTAG_PORT = 80
	#__FOXYTAG_USERNAME = 'test'
	#__FOXYTAG_PASSWORD = ''
	__FOXYTAG_LANGUAGE = 'en'
	__FOXYTAG_CLIENT = 'speedcamalerter'
	__FOXYTAG_VERSION = '1.0.0'
	__FOXYTAG_PLATFORM = 'NokiaN810'
	
	FOXYTAG_OK_INIT = 'OK_INIT'
	FOXYTAG_OK = 'OK'
	FOXYTAG_KO = 'KO'
	FOXYTAG_KO_CMD = 'KO_CMD'
	FOXYTAG_KO_VERSION = 'KO_VERSION'
	FOXYTAG_KO_LOGIN = 'KO_LOGIN'
	FOXYTAG_ERR = 'ERR'
	FOXYTAG_INF = 'INF'
	FOXYTAG_PUB = 'PUB'
	FOXYTAG_ERR_UNKNOWN = 'UNKNOW ERROR'
	FOXYTAG_ERR_UNEXPECTED = 'UNEXPECTED ERROR'

	##Constructor
	def __init__(self):
		self.__tags = []
		self.__username = ''
		self.__password = ''
		
	##Connect to server
	##@param username : username to use to connect to foxytag
	##@param password : password to use to connect to foxytag
	##@return res : result of tag post
	##@return usize : size of uploaded data
	##@return dsize : size of downloaded data
	##@return messages : messages returned by foxy server
	def connect(self, username, password):
		#init
		res = False
		usize = 0
		dsize = 0
		messages = []
		self.__username = username
		self.__password = password
	
		#prepare data to post
		data = urllib.urlencode([	('cmd' , 'init'), 
									('username' , self.__username),
									('password' , self.__password),
									('language' , self.__FOXYTAG_LANGUAGE),
									('client' , self.__FOXYTAG_CLIENT),
									('version' , self.__FOXYTAG_VERSION),
									('platform' , self.__FOXYTAG_PLATFORM) ])

		#get form
		try:
			url = self.__FOXYTAG_HOST+'?%s' % data
			handler = urllib.urlopen(url)
			lines = handler.read()
			#Log.log('CNX = %s' % lines)
			lines = lines.split('\n')
				
			#parse results
			(result, tag, messages) = self.__parseConnectionResults(lines)
			Log.log('CONNECT result=%s tag=%s msgs=%d' % (result, tag, len(messages)))
		
			#prepare output values
			res = result
			usize = len(url)
			dsize = len(lines)
			
		except Exception, e:
			#unable to connect to foxy server
			Log.log('CONNECT : unable to connect to foxy server [%s]' % str(e))
			res = False
			
		return res, usize, dsize, messages
			
	##Parse connection results
	def __parseConnectionResults(self, lines):
		#init
		result = False
		tag = self.FOXYTAG_ERR_UNEXPECTED
		messages = []
	
		#first line needs to be "MSG"
		print range(len(lines))
		for i in range(len(lines)):
			if i>=1 and len(lines[i].strip())>0:
				#print 'DATA = ',lines[i]
				#parse data
				if lines[i]==self.FOXYTAG_OK_INIT:
					result = True
					tag = self.FOXYTAG_OK_INIT
				elif lines[i]==self.FOXYTAG_KO:
					result = False
					tag = self.FOXYTAG_KO
				elif lines[i]==self.FOXYTAG_KO_CMD:
					result = False
					tag = self.FOXYTAG_KO_CMD
				elif lines[i]==self.FOXYTAG_KO_VERSION:
					result = False
					tag = self.FOXYTAG_KO_VERSION
				elif lines[i]==self.FOXYTAG_KO_LOGIN:
					result = False
					tag = self.FOXYTAG_KO_LOGIN
				elif lines[i].startswith(self.FOXYTAG_ERR):
					messages.append( FoxyMsg(self.FOXYTAG_ERR, lines[i]) )
				elif lines[i].startswith(self.FOXYTAG_INF):
					messages.append( FoxyMsg(self.FOXYTAG_INF, lines[i]) )
				elif lines[i].startswith(self.FOXYTAG_PUB):
					messages.append( FoxyMsg(self.FOXYTAG_PUB, lines[i]) )
				else:
					result = False
					tag = self.FOXYTAG_ERR_UNKNOWN
			elif i==0:
				#check header
				#print 'HEADER = ',lines[i]
				if lines[i]!='MSG':
					result = False
					tag = self.FOXYTAG_ERR_UNEXPECTED
					break
			
		return result, tag, messages
	
	##Get speedcams infos
	##@param lat : latitude
	##@param long : longitude
	##@param radius : radius (in meters, default 7250 meters)
	##@return res : result of tag post
	##@return usize : size of uploaded data
	##@return dsize : size of downloaded data
	##@return messages : messages returned by foxy server
	def tagRequest(self, lat, long, radius=7250):
		#init
		res = False
		usize = 0
		dsize = 0
		messages = []
		
		#prepare data to post
		data = urllib.urlencode([	('cmd' , 'tagrequest'), 
									('username' , self.__username),
									('password' , self.__password),
									('client' , self.__FOXYTAG_CLIENT),
									('lat' , lat), 
									('lon' , long), 
									('radius' , radius) ])

		try:
			#get form
			url = self.__FOXYTAG_HOST+'?%s' % data
			handler = urllib.urlopen(url)
			lines = handler.read()
			#Log.log('TAGREQ = %s' % lines)
			lines = lines.split('\n')
			
			#parse results
			(result, tag, messages) = self.__parseConnectionResults(lines)
			Log.log('TAGREQUEST : result=%s tag=%s msgs=%d' % (result, tag, len(messages)))
			if not result:
				#no result with connection parsing, try to parse tags
				(result, tag) = self.__parseTagRequestResults(lines)
				Log.log('TAGREQUEST : result=%s tag=%s' % (result, tag))
				if result:
					#display tags (debug)
					for i in range(len(self.__tags)):
						tag = self.__tags[i];
						#Log.log('lat=%.5f long=%.5f' % (tag.lat, tag.long))
					
					#prepare outputs
					res = True
					usize = len(url)
					dsize = len(lines)
					
				else:
					#unable to request tags
					res = False
			else:
				#connection result found, unable to request tags
				Log.log('TAGREQUEST : unable to connect to foxy server')
				res = False
		except:
			#unable to connect to foxy server
			res = False
			
		return res, usize, dsize, messages

	##Parse tag request results
	def __parseTagRequestResults(self, lines):
		#first line needs to be "TAG"
		if lines[0]=='TAG':
			#ok, clear current tags
			del self.__tags[0:len(self.__tags)]
			#second line is tag request, go ahead
			for i in range(2,len(lines)):
				if len(lines[i])>0:
					self.__tags.append( FoxyTag(lines[i]) )
			return (True, '')
		else:
			return (False, self.FOXYTAG_ERR_UNEXPECTED)
			
	##Get tags
	def getTags(self):
		return self.__tags
			
	##Send tag to server
	##@return res : result of tag post
	##@return usize : size of uploaded data
	##@return dsize : size of downloaded data
	def tagPost(self, tag, speed):
		#init
		res = False
		usize = 0
		dsize = 0
		
		if tag!=None and speed>20 and speed<1000:
			#prepare data to post
			data = urllib.urlencode([	('cmd' , 'tagpost'),
										('username' , self.__username),
										('password' , self.__password),
										('client' , self.__FOXYTAG_CLIENT),
										('lat' , tag.lat),
										('lon' , tag.long),
										('kind' , tag.kind),
										('heading' , tag.heading),
										('speed' , speed) ])
		
			try:
				#get form
				url = self.__FOXYTAG_HOST+'?%s' % data
				handler = urllib.urlopen(url)
				lines = handler.read()
				lines = lines.split('\n')
			
				#parse results
				(result, message) = self.__parseTagPostResults(lines)
				Log.log('TAGPOST : result=%s message=%s' % (result, message))
			
				res = True
				usize = len(url)
				dsize = len(lines)
			except:
				#unable to connect to foxy server
				Log.log('TAGPOST : unable to connect to foxy server')
				res = False
		else:
			#no tag specified
			res = False
			
		return res, usize, dsize
			
	##Parse connection results
	def __parseTagPostResults(self, lines):
		#first line needs to be "MSG"
		if lines[0]=='MSG':
			#ok, continu
			if lines[1]==self.FOXYTAG_OK:
				return (True, self.FOXYTAG_OK)
			elif lines[1]==self.FOXYTAG_KO:
				return (False, self.FOXYTAG_KO)
			elif lines[1]==self.FOXYTAG_KO_CMD:
				return (False, self.FOXYTAG_KO_CMD)
			elif lines[1]==self.FOXYTAG_KO_LOGIN:
				return (False, self.FOXYTAG_KO_LOGIN)
			elif lines[1].startswith(self.FOXYTAG_ERR):
				return (False, (lines[1][3:len(lines[1])]).strip())
			elif lines[1].startswith(self.FOXYTAG_INF):
				return (False, (lines[1][3:len(lines[1])]).strip())
			else:
				return (False, self.FOXYTAG_ERR_UNKNOWN)
		else:
			return (False, self.FOXYTAG_ERR_UNEXPECTED)
			
