from foxyradar import *
from foxytag import FoxyTag
from gtkeasyapp.easywindow import *
from gtkeasyapp.log import *
from gtkeasyapp.prefdialog import *
from gtkeasyapp.preffile import *
from devicemanager.connectionmanager import *
from gtkeasyapp.gis import *
from speedcamalerter import *

##version
VERSION = "1.0"

##Constants
path = os.path.dirname(__file__)
RESOURCES_DIR = os.path.join(path, 'res')

DOT_XPM = [
"10 10 2 1",
"  c black",
". c None",
"..........",
"...     ..",
"..       .",
".        .",
".         ",
".         ",
".        .",
".        .",
"..      ..",
"....  ...."
]

##SpeedcamAlerter menu
class SpeedcamAlerterMenu(gtk.Menu):
	##Constructor
	def __init__(self, pref):
		#super constructor
		gtk.Menu.__init__(self)
		
		#add debug item
		debug = pref.getItem('general', 'debug')
		if debug!=None and debug.getValue():
			#add speedcamalerter item
			speedcamalerter = gtk.MenuItem('SpeedcamAlerter')
			self.append(speedcamalerter)
		
			#add speedcamalerter sub menu
			speedcamalerter_sub = gtk.Menu()
			speedcamalerter.set_submenu(speedcamalerter_sub)
		
			#add speedcamalerter start sub items
			self.item_speedcamalerter_start = gtk.MenuItem('Start')
			speedcamalerter_sub.append(self.item_speedcamalerter_start)
		
			#add speedcamalerter stop sub items
			self.item_speedcamalerter_stop = gtk.MenuItem('Stop')
			speedcamalerter_sub.append(self.item_speedcamalerter_stop)
			
			#add separator
			self.append( gtk.SeparatorMenuItem() )
		
		#add preferences item
		self.item_preferences = gtk.MenuItem('Preferences')
		self.append(self.item_preferences)
		
		#add separator
		self.append( gtk.SeparatorMenuItem() )
		
		#add help item
		self.item_help = gtk.MenuItem('Help')
		self.append(self.item_help)
		
		#add about item
		self.item_about = gtk.MenuItem('About')
		self.append(self.item_about)
		
		#add separator
		self.append( gtk.SeparatorMenuItem() )
		
		#add exit item
		self.item_exit = gtk.MenuItem('Exit')
		self.append(self.item_exit)
		
##SpeedcamAlerter Preferences dialog
class SpeedcamAlerterPreferences(PrefDialog):
	##Constructor
	def __init__(self, parent, prefFile):
		#super constructor
		PrefDialog.__init__(self, parent, 'Preferences')
		self.set_position(gtk.WIN_POS_CENTER)
		
		#load items from pref file
		prefItemDebug = prefFile.getItem('general', 'debug')
		prefItemGps = prefFile.getItem('general', 'gps')
		prefItemFront = prefFile.getItem('general', 'front')
		prefItemFixed = prefFile.getItem('general', 'fixed')
		prefItemMobile = prefFile.getItem('general', 'mobile')
		prefItemGhost = prefFile.getItem('general', 'ghost')
		prefItemRadius = prefFile.getItem('display', 'radius')
		prefItemEco = prefFile.getItem('display', 'eco')
		prefItemConnection = prefFile.getItem('connections', 'connection')
		prefItemFoxyLogin = prefFile.getItem('connections', 'login')
		prefItemFoxyPassword = prefFile.getItem('connections', 'password')
		prefItemDownload = prefFile.getItem('connections', 'download')
		prefItemUpload = prefFile.getItem('connections', 'upload')
		prefItemDummyDownload = PrefFileItemDummy()
		prefItemDummyUpload = PrefFileItemDummy()
		
		#add sections
		self.addSection('general', 'General', os.path.join(RESOURCES_DIR, 'section_general.png'))
		self.addSection('display', 'Display', os.path.join(RESOURCES_DIR, 'section_display.png'))
		self.addSection('connections', 'Connections', os.path.join(RESOURCES_DIR, 'section_connections.png'))
		
		#create pref dialog items
		itemFront = PrefDialogItemTrueFalse(prefItemFront, 'Bring to front')
		itemFront.setHelp('Bring to front SpeedcamAlerter application when necessary')
		self.addItem('general', itemFront)
		itemFixed = PrefDialogItemInput(prefItemFixed, 'Fixed alert at (m)')
		itemFixed.setHelp('Distance at which the alarm is triggered when fixed speedcam is close to the car')
		self.addItem('general', itemFixed)
		itemMobile = PrefDialogItemInput(prefItemMobile, 'Mobile alert at (m)')
		itemMobile.setHelp('Distance at which the alarm is triggered when mobile speedcam is close to the car')
		self.addItem('general', itemMobile)
		itemGhost = PrefDialogItemInput(prefItemGhost, 'Ghost alert at (m)')
		itemGhost.setHelp('Distance at which the alarm is triggered when ghost speedcam is close to the car')
		self.addItem('general', itemGhost)
		
		itemRadius = PrefDialogItemPlusMinus(prefItemRadius, 'Radar radius (km)', 3, 6)
		itemRadius.setHelp('Area covered by the radar')
		self.addItem('display', itemRadius)
		itemEco = PrefDialogItemTrueFalse(prefItemEco, 'Turns off screen')
		itemEco.setHelp('Turns on screen only when necessary to save power')
		self.addItem('display', itemEco)
		
		itemConnection = PrefDialogItemList(prefItemConnection, 'Data connection')
		itemConnection.setHelp('Data connection to use to connect to internet and retrieve foxy tags')
		self.addItem('connections', itemConnection)
		itemSeparator1 = PrefDialogItemSeparator('<b>FoxyTag params</b>')
		self.addItem('connections', itemSeparator1)
		itemLogin = PrefDialogItemInput(prefItemFoxyLogin, 'Login')
		self.addItem('connections', itemLogin)
		itemPassword = PrefDialogItemPassword(prefItemFoxyPassword, 'Password')
		self.addItem('connections', itemPassword)
		itemFoxyTagInscription = PrefDialogItemLink('FoxyTag webpage', 'http://www.foxytag.com/fr/download.html', 'http://www.foxytag.com')
		itemFoxyTagInscription.setHelp('All information about FoxyTag registration is specified at bottom of webpage')
		self.addItem('connections', itemFoxyTagInscription)
		itemSeparator1 = PrefDialogItemSeparator('<b>Data counters</b>')
		self.addItem('connections', itemSeparator1)
		download = float(prefItemDownload.getValue() * 0.000001)
		prefItemDummyDownload.setValue('%.2f' % download)
		itemDownload = PrefDialogItemInput(prefItemDummyDownload, 'Download (Mb)')
		itemDownload.setHelp('Amount of data downloaded from foxytag server')
		itemDownload.setReadOnly(True)
		self.addItem('connections', itemDownload)
		upload = float(prefItemUpload.getValue() * 0.000001)
		prefItemDummyUpload.setValue('%.2f' % upload)
		itemUpload = PrefDialogItemInput(prefItemDummyUpload, 'Upload (Mb)')
		itemUpload.setHelp('Amount of data uploaded to foxytag server')
		itemUpload.setReadOnly(True)
		self.addItem('connections', itemUpload)
		
		#add data connections
		connections = ConnectionManager.getConnections(ConnectionItem.CNX_TYPE_DATA)
		options = ['']
		for connection in connections:
			options.append(connection.name)
		itemConnection.setOptions(options)
		
##SpeedcamAlerter about dialog
class SpeedcamAlerterAboutDialog(gtk.Dialog):
	##Constructor
	def __init__(self, parent):
		#super constructor
		gtk.Dialog.__init__(self, 'About', parent, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
		self.set_default_size(400, 250)
		self.set_position(gtk.WIN_POS_CENTER)
		self.set_has_separator(False)
		
		#add foxy tag image
		image = gtk.Image()
		image.set_from_file(os.path.join(RESOURCES_DIR, 'foxytag.png'))
		self.vbox.pack_start(image)
		
		#add speedcamalerter infos (text is extracted from eyekiwi website)
		label = gtk.Label('SpeedcamAlerter uses FoxyTag, the world largest speed camera database. The collaborative structure of FoxyTag offers you continuously the most accurate and always up to date speed camera location data.')
		label.set_line_wrap(True)
		self.vbox.pack_start(label)
		
		self.show_all()

##HelpDialogLabel build pre-defined label widget
class HelpDialogLabel(gtk.HBox):
	##Constructor
	def __init__(self, window, label, color=None, addDot=True):
		#super constructor
		gtk.HBox.__init__(self)
		
		#add dot
		if addDot:
			xpm,mask = gtk.gdk.pixmap_create_from_xpm_d(window.window, None, DOT_XPM)
			dot = gtk.Image()
			dot.set_from_pixmap(xpm, mask)
			#dot.set_padding(5, 5)
			dot.set_size_request(10, 10)
			self.pack_start(dot, False, False, 5)
		
		#set attributes
		lbl = gtk.Label()
		lbl.set_line_wrap(True)
		lbl.set_selectable(False)
		#lbl.set_padding(2, 2)
		lbl.set_alignment(0.0, 0.5)
		if color!=None:
			lbl.set_markup('<span foreground="%s">%s</span>' % (color,label) )
		else:
			lbl.set_markup('%s' % (label) )
		self.pack_start(lbl, True, True, 5)
			
			
			
##HelpDialogImageLabel build image-label widget
class HelpDialogImageLabel(gtk.HBox):
	##Constructor
	def __init__(self, window, label, image, addDot=True):
		#super constructor
		gtk.HBox.__init__(self)
		
		#add dot
		if addDot:
			xpm,mask = gtk.gdk.pixmap_create_from_xpm_d(window.window, None, DOT_XPM)
			dot = gtk.Image()
			dot.set_from_pixmap(xpm, mask)
			#dot.set_padding(5, 5)
			dot.set_size_request(10, 10)
			self.pack_start(dot, False, False, 5)
		
		#add image
		pixbuf = gtk.gdk.pixbuf_new_from_file(image)
		image = gtk.Image()
		image.set_from_pixbuf(pixbuf)
		#image.set_padding(5, 5)
		image.set_size_request(pixbuf.get_width(), pixbuf.get_height())
		self.pack_start(image, False, False, 5)

		#add label
		label = gtk.Label(label)
		label.set_line_wrap(True)
		#label.set_padding(2, 2)
		label.set_selectable(False)
		label.set_alignment(0.0, 0.5)
		self.pack_start(label, True, True)
		
		
		
		
##SpeedcamAlerter help dialog
class SpeedcamAlerterHelpDialog(gtk.Dialog):
	##Constructor
	def __init__(self, parent):
		#Super constructor
		gtk.Dialog.__init__(self, 'Help', parent, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
		self.set_default_size(650, 325)
		self.set_has_separator(False)
		self.set_position(gtk.WIN_POS_CENTER)
		scrolled = gtk.ScrolledWindow()
		scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		self.vbox.pack_start(scrolled)
		vbox = gtk.VBox(spacing=5)
		scrolled.add_with_viewport(vbox)
		
		#add instructions label
		instructions = HelpDialogLabel(parent, '<b><u>FoxyTag instructions:</u></b>', None, False)
		vbox.pack_start(instructions)

		#add instructions
		instruction1 = HelpDialogLabel(parent, 'User must tag or confirm when he is as close as possible to the speed camera and not already as soon as he sees the camera. Otherwise there will be a second tag for the same camera and other users will decrease his trust links.', 'red')
		vbox.pack_start(instruction1)
		instruction2 = HelpDialogLabel(parent, 'A camera inside a tunnel or close to a tunnel exit (typically  less than 10 seconds after the exit) must be tagged at the tunnel entry.', 'red')
		vbox.pack_start(instruction2)
		instruction3 = HelpDialogLabel(parent, 'When the tagging needs several user-interactions (like first choosing the kind of camera and then choosing its direction), it is when he presses the first button that the current position is recorded. He can therefore take as much time as he needs to finish the tagging process.', 'red')
		vbox.pack_start(instruction3)
		instruction4 = HelpDialogLabel(parent, 'It is useless to confirm several times the same speed camera. In some particular situations this can even be bad for his trust links.', 'orange')
		vbox.pack_start(instruction4)
		instruction5 = HelpDialogLabel(parent, 'In case of a doubt it is better not to tag instead of tagging wrong. Many users think that if they forget to tag or to confirm a speed camera their trust links will decrease, which is wrong.', 'orange')
		vbox.pack_start(instruction5)
		instruction6 = HelpDialogLabel(parent, 'If he misuses the application (like confirming a speed camera when he is not close enough) his trust links will decrease and he risks therefore to be excluded from the system.')
		vbox.pack_start(instruction6)
		instruction7 = HelpDialogLabel(parent, 'If there can be a doubt whether a camera can flash in both directions or not, it is better to tag for both directions.')
		vbox.pack_start(instruction7)
		instruction8 = HelpDialogLabel(parent, 'A traffic light camera should be tagged and treated like a speed camera.')
		vbox.pack_start(instruction8)
		
		#add separator
		separator = gtk.Label('')
		vbox.pack_start(separator)

		#add help label
		helps = HelpDialogLabel(parent, '<b><u>SpeedcamAlerter help:</u></b>', None, False)
		vbox.pack_start(helps)
		
		#add helps
		help1 = HelpDialogLabel(parent, 'Tap on the radar to add, confirm or cancel speedcam. Then follow screen instructions.')
		vbox.pack_start(help1)
		help2 = HelpDialogLabel(parent, 'When radar approaching, screen turns on automatically.')
		vbox.pack_start(help2)
		help3 = HelpDialogImageLabel(parent, 'Red logo represents fixed speedcam in car direction.', os.path.join(RESOURCES_DIR, 'speedcam_fixed_40x40.png'))
		vbox.pack_start(help3)
		help4 = HelpDialogImageLabel(parent, 'Blue logo represents mobile speedcam in car direction.', os.path.join(RESOURCES_DIR, 'speedcam_mobile_40x40.png'))
		vbox.pack_start(help4)
		help5 = HelpDialogImageLabel(parent, 'Grey logo represents ghost speedcam in car direction.', os.path.join(RESOURCES_DIR, 'speedcam_ghost_40x40.png'))
		vbox.pack_start(help5)
		help6 = HelpDialogImageLabel(parent, 'Green logo represents speedcam in opposite car direction.', os.path.join(RESOURCES_DIR, 'speedcam_opposite_40x40.png'))
		vbox.pack_start(help6)
		
		self.show_all()
		
		
		
		
##SpeedcamAlerter User Interface
class SpeedcamAlerterUI(EasyWindow):
	##Constructor
	def __init__(self, pref):
		#get preferences filepath
		self.speedcamalerter = None
	
		#super constructor
		EasyWindow.__init__(self, 'SpeecamAlerter', pref, False)
	
	
	##Get old version (version during last launch, usualy read from preferences file)
	##@return None if no version or error occured
	def _getOldVersion(self, prefFile=None):
		if prefFile!=None:
			version = prefFile.getItem('general', 'version')
			if version!=None:
				return version.getValue()
		return None
		
		
	##Get new version (usualy read from application global variable)
	##@return None if no version or error occured
	def _getNewVersion(self, prefFile=None):
		return VERSION
	
	
	##Create new preferences file
	def _createDefaultPreferencesFile(self, filePath):
		#prepare xml content
		xml =  '<config>\n'
		xml += '  <section id="general">\n'
		xml += '    <key id="gps" type="bool">1</key>\n'
		xml += '    <key id="front" type="bool">1</key>\n'
		xml += '    <key id="fixed" type="int">800</key>\n'
		xml += '    <key id="mobile" type="int">800</key>\n'
		xml += '    <key id="ghost" type="int">500</key>\n'
		xml += '    <key id="version" type="string"></key>\n'
		xml += '  </section>\n'
		xml += '  <section id="display">\n'
		xml += '    <key id="radius" type="int">4</key>\n'
		xml += '    <key id="eco" type="bool">1</key>\n'
		xml += '  </section>\n'
		xml += '  <section id="connections">\n'
		xml += '    <key id="connection" type="string"></key>\n'
		xml += '    <key id="download" type="int">0</key>\n'
		xml += '    <key id="upload" type="int">0</key>\n'
		xml += '    <key id="login" type="string"></key>\n'
		xml += '    <key id="password" type="string"></key>\n'
		xml += '  </section>\n'
		xml += '</config>'
		
		#write xml
		conf = open(filePath, 'wb')
		conf.write(xml)
		conf.close()
		
	
	##Check preferences file after new version detected (useful to add or remove item)
	def _checkPreferencesFile(self, filePath):
		#no parameter to add
		pass
		
	
	##Create application menu
	##@param window : main gtk window
	##@param prefFile : PrefFile object
	##@return menu : gtk.Menu object
	def _createMenu(self, window, prefFile):
		menu = SpeedcamAlerterMenu(self._prefFile)
		debug = prefFile.getItem('general', 'debug')
		if debug!=None and debug.getValue():
			menu.item_speedcamalerter_start.connect('activate', self.__startSpeedcamAlerter)
			menu.item_speedcamalerter_stop.connect('activate', self.__stopSpeedcamAlerter)
		menu.item_preferences.connect('activate', self.__preferences)
		menu.item_help.connect('activate', self.__help)
		menu.item_about.connect('activate', self.__about)
		menu.item_exit.connect('activate', self._quit)
		return menu
	
	
	##Add main container
	##@param window : main gtk window
	##@param prefFile : PrefFile object
	##@return main GTK container
	def _createMainContainer(self, window, prefFile):
		#create main grid
		container = gtk.Table(4, 2, False)
		return container
	
	
	##Create widgets
	##@param window : main gtk window
	##@param prefFile : PrefFile object
	##@return nothing, just create all body widgets
	def _createWidgets(self, window, prefFile):
		#add foxytag logo
		self.__image_logo = gtk.Image()
		self.__image_logo.set_from_file(os.path.join(RESOURCES_DIR, 'foxytag.png'))
		self._mainContainer.attach(self.__image_logo, 1, 2, 0, 1, gtk.FILL, gtk.FILL,5 ,5)
		
		#add speed widget
		self.__label_speed = gtk.Label('speed')
		self.__label_speed.set_alignment(0.5, 0.5)
		self.__label_speed.modify_font(pango.FontDescription("sans 40"))
		label_speed_event = gtk.EventBox()
		label_speed_event.add(self.__label_speed)
		#label_speed_event.modify_bg(gtk.STATE_NORMAL, label_speed_event.get_colormap().alloc_color("green"))
		self._mainContainer.attach(label_speed_event, 1, 2, 1, 2, gtk.FILL, gtk.FILL, 0, 0)
		self.__setSpeed(0)
		
		#add distance hbox 
		hbox = gtk.HBox()
		self._mainContainer.attach(hbox, 1, 2, 2, 3, gtk.FILL, gtk.FILL, 0, 0)
		
		#add distance image
		self.__image_speedcam_fixed = gtk.gdk.pixbuf_new_from_file(os.path.join(RESOURCES_DIR, 'speedcam_fixed_50x50.png'))
		self.__image_speedcam_mobile = gtk.gdk.pixbuf_new_from_file(os.path.join(RESOURCES_DIR, 'speedcam_mobile_50x50.png'))
		self.__image_speedcam_ghost = gtk.gdk.pixbuf_new_from_file(os.path.join(RESOURCES_DIR, 'speedcam_ghost_50x50.png'))
		self.__image_speedcam_opposite = gtk.gdk.pixbuf_new_from_file(os.path.join(RESOURCES_DIR, 'speedcam_opposite_50x50.png'))
		self.__image_speedcam = gtk.Image()
		self.__image_speedcam.set_padding(5,5)
		hbox.pack_start(self.__image_speedcam, False, True, 8)
		
		#add distance widget
		self.__label_distance = gtk.Label('')
		self.__label_distance.set_alignment(0, 0.5)
		self.__label_distance.modify_font(pango.FontDescription("sans 40"))
		label_distance_event = gtk.EventBox();
		label_distance_event.add(self.__label_distance)
		hbox.pack_start(label_distance_event, False, False, 0)
		self.__setDistance(None)
				
		#add message widget
		self.__label_message = gtk.Label('')
		self.__label_message.modify_font(pango.FontDescription("sans 20"))
		self.__label_message.set_single_line_mode(False) #enable multiline only for message
		self.__label_speed.set_alignment(0.5, 0.5)
		label_message_event = gtk.EventBox()
		label_message_event.add(self.__label_message)
		self._mainContainer.attach(label_message_event, 0, 2, 3, 4, 0, 0, 10, 10)
		self.__setMessage('')
		
		#add radar widget
		self.__radar = FoxyRadar()
		radius = prefFile.getItem('display', 'radius')
		if radius!=None:
			self.__radar.setRadius(radius.getValue())
		self._mainContainer.attach(self.__radar, 0,1,0,3, gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 2, 2)
		self.__radar.connect('add-fixed-speedcam', self.__addFixedSpeedcam)
		self.__radar.connect('add-opposite-fixed-speedcam', self.__addOppositeFixedSpeedcam)
		self.__radar.connect('add-mobile-speedcam', self.__addMobileSpeedcam)
		self.__radar.connect('add-opposite-mobile-speedcam', self.__addOppositeMobileSpeedcam)
		self.__radar.connect('remove-speedcam', self.__removeSpeedcam)
		self.__radar.connect('action-canceled', self.__radarActionCanceled)
		self.__radar.connect('action-started', self.__radarActionStarted)
		
		
	##Startup treatment (like open splash screen...)
	##@param window : main gtk window
	##@param prefFile : PrefFile object
	##@return nothing
	def _startupTreatment(self, window, prefFile):
		#create speedcamalerter 
		#pref, setSpeedCallback, setDistanceCallback, setMessageCallback, setPositionCallback, addFoxyTagCallback, removeAllFoxyTagsCallback, setSpeedcamIsCloseCallback
		self.speedcamalerter = SpeedcamAlerter(prefFile, self.__setSpeed, self.__setDistance, self.__setMessage, self.__setPosition, self.__addFoxyTag, self.__removeAllFoxyTags, self.__setSpeedcamIsClose)
		
		#check preferences values
		firstLaunch = False
		startSpeedcamAlerter = True
		version = prefFile.getItem('general', 'version')
		gps = prefFile.getItem('general', 'gps')
		login = prefFile.getItem('connections', 'login')
		
		#check gps value (used in debug only)
		if gps!=None and gps.getValue():
			startSpeedcamAlerter = False
			
		#check version
		if version!=None and version.getValue()!=VERSION:
			if len(version.getValue())==0:
				#first run
				firstLaunch = True
				self.__firstLaunch()
				startSpeedcamAlerter = False
			else:
				#new version
				pass
			version.setValue(VERSION)
			prefFile.save()
			
		#check login
		if not firstLaunch and login!=None and len(login.getValue())==0:
			self.__noAccount()
			startSpeedcamAlerter = False
			
		#start speedcamalerter
		if startSpeedcamAlerter:
			self.__startSpeedcamAlerter()
		
		
	##first launch info
	def __firstLaunch(self, *args):
		dialog = gtk.MessageDialog(self._window, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, 'It\'s the first launch, please go to preferences\ndialog and configure FoxyTag account, SpeedcamAlerter\nneeds this to work. But don\'t worry it\'s free :-)')
		dialog.run()
		dialog.destroy()
	
	
	##Message to display if no foxytag account specified
	def __noAccount(self, *args):
		dialog = gtk.MessageDialog(self._window, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, 'SpeedcamAlerter needs FoxyTag account to start (it\'s free),\nplease configure it in preferences dialog.')
		dialog.run()
		dialog.destroy()
		
		
	##Open about dialog
	def __about(self, *args):
		dialog = SpeedcamAlerterAboutDialog(self._window)
		dialog.run()
		dialog.destroy()
		
		
	##Open help dialog
	def __help(self, *args):
		dialog = SpeedcamAlerterHelpDialog(self._window)
		dialog.run()
		dialog.destroy()
	
	
	##Open preferences dialog
	def __preferences(self, *args):
		#init
		passwordChanged = False
		loginChanged = False
		connectionChanged = False
		password = ""
		login = ""
		connection = ""
		
		#save foxytag login before opening preferences
		loginItem = self._prefFile.getItem('connections', 'login')
		if loginItem!=None:
			login += str(loginItem.getValue()) #make a copy
	
		#save foxytag password before opening preferences
		passwordItem = self._prefFile.getItem('connections', 'password')
		if passwordItem!=None:
			password += str(passwordItem.getValue()) #make a copy
			
		#save connection before opening preferences
		connectionItem = self._prefFile.getItem('connections', 'connection')
		if connectionItem!=None:
			connection += str(connectionItem.getValue()) #make a copy
	
		#open preferences
		dialog = SpeedcamAlerterPreferences(self._window, self._prefFile)
		response = dialog.run()
		if response==gtk.RESPONSE_OK:
			#save changed config
			self._prefFile.save()
			
			#update radius
			radius = self._prefFile.getItem('display', 'radius')
			if radius!=None:
				self.__radar.setRadius(radius.getValue())
				
			#test mode
			passwordItem = self._prefFile.getItem('connections', 'password')
			if passwordItem!=None:
				if len(passwordItem.getValue())==0:
					self.__radar.setTestMode(True)
				else:
					self.__radar.setTestMode(False)

			#new password ?
			if passwordItem!=None and passwordItem.getValue()!=password:
				#user changed password, display help
				passwordChanged = True
				
			#new login ?
			loginItem = self._prefFile.getItem('connections', 'login')
			if loginItem!=None and loginItem.getValue()!=login:
				#login changed, need to restart speedcamalerter
				loginChanged = True
				
			#new connection ?
			connectionItem = self._prefFile.getItem('connections', 'connection')
			if connectionItem!=None and connectionItem.getValue()!=connection:
				#connection changed, need to restart speedcamalerter
				connectionChanged = True
		
		#destroy preferences dialog
		dialog.destroy()
		
		#display help if password modified
		if passwordChanged:
			self.__help()
			
		#restart speedcamalerter if login, password or connection changed
		if loginChanged or passwordChanged or connectionChanged:
			self.__restartSpeedcamAlerter()


	##Before quit callback
	def _beforeQuit(self):			
		#stop speedcamalerter
		self.__stopSpeedcamAlerter()
		
	
	##Radar action is canceled
	def __radarActionCanceled(self, widget, data=None):
		if self.speedcamalerter!=None:
			self.speedcamalerter.resetActionPosition()
			
			
	##Radar view is clicked
	def __radarActionStarted(self, widget, data=None):
		if self.speedcamalerter!=None:
			self.speedcamalerter.saveActionPosition()
	
	
	##Remove fixed speedcam callback
	def __addFixedSpeedcam(self, widget, data=None):
		if self.speedcamalerter!=None:
			self.speedcamalerter.addFixedSpeedcam()
	
	
	##Remove opposite fixed speedcam callback
	def __addOppositeFixedSpeedcam(self, widget, data=None):
		if self.speedcamalerter!=None:
			self.speedcamalerter.addOppositeFixedSpeedcam()
	
	
	##Remove mobile speedcam callback
	def __addMobileSpeedcam(self, widget, data=None):
		if self.speedcamalerter!=None:
			self.speedcamalerter.addMobileSpeedcam()
	
	
	##Add opposite mobile speedcam callback
	def __addOppositeMobileSpeedcam(self, widget, data=None):
		if self.speedcamalerter!=None:
			self.speedcamalerter.addOppositeMobileSpeedcam()
	
	
	##Remove speedcam callback
	def __removeSpeedcam(self, widget, data=None):
		if self.speedcamalerter!=None:
			self.speedcamalerter.removeSpeedcam()
	
	
	##Start speedcamalerter
	def __startSpeedcamAlerter(self, widget=None, data=None):
		if self.speedcamalerter!=None and self.speedcamalerter.isStopped():
			self.speedcamalerter.start()
	
	
	##Stop speedcamalerter
	def __stopSpeedcamAlerter(self, widget=None, data=None):
		if self.speedcamalerter!=None and not self.speedcamalerter.isStopped():
			self.speedcamalerter.stop()
			
			
	##Restart SpeedcamAlerter
	def __restartSpeedcamAlerter(self, widget=None, data=None):
		#stop
		self.__stopSpeedcamAlerter()
		
		#wait 2 seconds and start speedcamalerter
		gobject.timeout_add(2, self.__startSpeedcamAlerter)
	
	
	##Set speedcam is close to the car
	##@param isClose : True if the car is close to a speedcam
	##@param speedcamDistance : distance between car and speedcam
	def __setSpeedcamIsClose(self, isClose, speedcamDistance):
		gtk.gdk.threads_enter()
		try:
			#update radar
			self.__radar.speedcamIsClose(isClose, speedcamDistance)
			
			#bring window to front
			if isClose:
				front = self.__pref.getItem('general', 'front')
				if front!=None and front.getValue():
					self.bringToFront()
					
		except Exception,e:
			Log.log(e.message)
		gtk.gdk.threads_leave()
		
		
	##Set radar current position
	##@param utmPoint : current utm position
	##@param speed : current speed in km/h
	##@param degAngle : current angle in degrees
	def __setPosition(self, utmPoint, speed, degAngle):
		gtk.gdk.threads_enter()
		try:
			self.__radar.setPosition(utmPoint, speed, degAngle)
		except Exception,e:
			Log.log(e.message)
		gtk.gdk.threads_leave()
	
	
	##Add new foxy tag to radar
	##@param tag : tag to add
	def __addFoxyTag(self, tag):
		gtk.gdk.threads_enter()
		try:
			self.__radar.addFoxyTag(tag)
		except Exception,e:
			Log.log(e.message)
		gtk.gdk.threads_leave()
		
		
	##Remove all foxy tags from radar
	def __removeAllFoxyTags(self):
		gtk.gdk.threads_enter()
		#Log.log('enter removeallfoxytags')
		try:
			self.__radar.removeAllFoxyTags()
		except Exception,e:
			Log.log(e.message)
		#Log.log('leave removeallfoxytags')
		gtk.gdk.threads_leave()
	
	
	##Change UI message
	##@param message : new message
	def __setMessage(self, message):
		gtk.gdk.threads_enter()
		try:
			#get new text pixel width
			layout = self.__label_message.get_layout()
			context = layout.get_context()
			newLayout = pango.Layout(context)
			newLayout.set_font_description(pango.FontDescription("sans 20"))
			newLayout.set_text(message)
		
			#compare new message pixel width to window width
			continu = True
			while continu:
				if newLayout.get_pixel_size()[0]>self._window.get_size()[0]:
					#text seems to be too large, remove 10 chars
					message = message[0:len(message)-5] + '...'
					newLayout.set_text(message)
				else:
					continu = False

			#set message
			self.__label_message.set_label(message)
		except Exception,e:
			Log.log(e.message)
		gtk.gdk.threads_leave()
	
	
	##Change UI speed
	##@param speed : current speed in km/h
	def __setSpeed(self, speed):
		gtk.gdk.threads_enter()
		try:
			speed = '%.0fkm/h' % speed
			self.__label_speed.set_label(str(speed))
		except Exception,e:
			Log.log(e.message)
		gtk.gdk.threads_leave()
	
	
	##Change UI distance
	##@param foxyTagForSpeedcamAlerter : foxytag for speedcam alerter object
	def __setDistance(self, foxyTagForSpeedcamAlerter):
		gtk.gdk.threads_enter()
		try:
			if foxyTagForSpeedcamAlerter!=None:
				#init
				kind = foxyTagForSpeedcamAlerter.getFoxyTag().kind
				opposite = foxyTagForSpeedcamAlerter.getFoxyTag().opposite
				distance = foxyTagForSpeedcamAlerter.getDistanceFromCar()
			
				#update distance
				if distance>6000 or distance==0:
					#speedcam too far or no distance specified
					self.__label_distance.set_label('-')
				else:
					#update speedcam logo
					if kind==None:
						#remove image
						self.__image_speedcam.clear()
					else:
						#add image according to speedcam kind
						if opposite:
							self.__image_speedcam.set_from_pixbuf(self.__image_speedcam_opposite)
						elif kind==FoxyTag.FOXYTAG_FIXED:
							self.__image_speedcam.set_from_pixbuf(self.__image_speedcam_fixed)
						elif kind==FoxyTag.FOXYTAG_MOBILE:
							self.__image_speedcam.set_from_pixbuf(self.__image_speedcam_mobile)
						elif kind==FoxyTag.FOXYTAG_GHOST:
							self.__image_speedcam.set_from_pixbuf(self.__image_speedcam_ghost)
				
					#update distance
					if distance>1000:
						#distance in km
						distance = float(distance)/1000.0
						self.__label_distance.set_label('%.2fkm' % distance)
					else:
						#distance in m
						self.__label_distance.set_label('%dm' % distance)
		except Exception,e:
			Log.log(e.message)
		gtk.gdk.threads_leave()
